using System;
using System.IO;
using System.Windows.Forms;
using System.Collections;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Configuration.Install;
using System.Data;
using System.Data.SqlClient;
using System.Net;
using System.Reflection;
using System.Xml;
using Microsoft.Win32;
using System.EnterpriseServices.Internal;

[RunInstaller(true)]
public class CustomInstaller : System.Configuration.Install.Installer
{
    private static Installer.wsEvents.Events events = new Installer.wsEvents.Events();
    private const string CONFIG_PATH = @"C:\Program Files\Microsoft Office Servers\12.0\Bin\"; // reading the value from registry
    private const string CONFIG_FILE = "Microsoft.Office.Project.Server.Eventing.exe.config";
    private const string EVENT_HANDLER_ASSEMBLY = "Microsoft.EPM.Events.CubeEvent.dll";

    private string getOfficeSystemConfig_Path()
    {
        try
        {

            RegistryKey HKLM = RegistryKey.OpenRemoteBaseKey(RegistryHive.LocalMachine, "");
            string subkey = "SOFTWARE\\Microsoft\\Office Server\\12.0";
            RegistryKey OfficeSubkey = HKLM.OpenSubKey(subkey);
            string result = OfficeSubkey.GetValue("BinPath").ToString();
            if (result.Length == 0)
            {
                result = CONFIG_PATH; // this case may happen on 64 bits systems 
            }
            return result;
        }
        catch (Exception ex)
        {
            // this case may happen on 64 bits systems 
            return CONFIG_PATH;
        }
    }

    public CustomInstaller() : base()
    {
    }
   
    public override void Install(IDictionary savedState)
    {
        try
        {
            // Get the parameters.
            string url = Context.Parameters["URL"];
            string logFolderPath = Context.Parameters["LOGFOLDERPATH"];
            string logFileEveryHour = Context.Parameters["LOGFILEEVERYHOUR"];
            string logAdminEvent = Context.Parameters["LOGADMINEVENT"];

            // register the assembly in GAC

            RegisterAssemblyInGAC();

            // Register the event handlers.
            StringCollection eventUids = null;
            eventUids = RegisterEventHandler(url);
            // Update the events service config with the config params for the handler.

            if (!logFolderPath.EndsWith("\\")) logFolderPath = logFolderPath+"\\";

            UpdateEventsConfig(logFolderPath,logFileEveryHour, logAdminEvent);

            // Save the parameters for the uninstall.
            savedState.Add("URL", url);
            savedState.Add("EVENTUIDS", eventUids);

            base.Install(savedState);
        }
        catch (Exception ex)
        {
            throw new InstallException(ex.Message);
        }
    }
   
    public override void Uninstall(IDictionary savedState)
    {
        try
        {
            // unregister the assembly from GAC

            UnRegisterAssemblyInGAC();
            
            RemoveEventHandlers(savedState["URL"].ToString(), (StringCollection)savedState["EVENTUIDS"]);

            base.Uninstall(savedState);
        }
        catch (Exception ex)
        {
            throw new InstallException(ex.Message);
        }
    }

    private void RegisterAssemblyInGAC()
    {
        string assem = GetInstallDir() + "\\" + EVENT_HANDLER_ASSEMBLY;
        Publish p = new Publish();

        p.GacInstall(assem);

    }

    private void UnRegisterAssemblyInGAC()
    {
        string assem = GetInstallDir() + "\\" + EVENT_HANDLER_ASSEMBLY;
        Publish p = new Publish();

        p.GacRemove(assem);

    }
   
    private StringCollection RegisterEventHandler(string url)
    {
        try
        {
            // Set the url for the events web service.
            if (!url.EndsWith("/")) url = url + "/";
            events.Url = url + "_vti_bin/psi/events.asmx";
            events.Credentials = CredentialCache.DefaultCredentials;

            // Get the event info from the assembly.
            string assem = GetInstallDir() + "\\" + EVENT_HANDLER_ASSEMBLY;
            Assembly eventAssem = Assembly.LoadFile(assem);


            Installer.wsEvents.EventHandlersDataSet eventHandlers = new Installer.wsEvents.EventHandlersDataSet();

            StringCollection eventUids = new StringCollection();
            foreach (Type t in eventAssem.GetTypes())
            {
                // Make sure it is an event receiver class.
                if (t.IsClass && t.BaseType.Module.Name == "Microsoft.Office.Project.Server.Events.Receivers.dll")
                {
                    foreach (MethodInfo mi in t.GetMethods())
                    {
                        if (mi.DeclaringType.FullName == t.FullName)
                        {
                            Installer.wsEvents.EventHandlersDataSet.EventHandlersRow row = eventHandlers.EventHandlers.NewEventHandlersRow();
                            row.AssemblyName = eventAssem.FullName;
                            row.ClassName = t.FullName;
                            row.Description = "Added by installer.";
                            row.EventHandlerUid = Guid.NewGuid();
                            row.EventId = GetEventID(t.BaseType.Name, mi.Name);
                            row.Name = t.BaseType.Name + " " + mi.Name + " Cube Event Handler";
                            row.Order = 1;
                            eventHandlers.EventHandlers.AddEventHandlersRow(row);
                            eventUids.Add(row.EventHandlerUid.ToString());
                        }
                    }
                }
            }
            events.CreateEventHandlerAssociations(eventHandlers);
            return eventUids;
        }
        catch (Exception ex)
        {
            MessageBox.Show("Event handler registration failed.\n" + ex.Message);
            return null;
        }
    }

    private void RemoveEventHandlers(string url, StringCollection eventUids)
    {
        if (eventUids == null) return;

        try
        {
            if (!url.EndsWith("/")) url = url + "/";
            events.Url = url + "_vti_bin/psi/events.asmx";
            events.Credentials = CredentialCache.DefaultCredentials;

            Guid[] arr = new Guid[eventUids.Count];
            int i = 0;
            foreach (string uid in eventUids)
            {
                arr[i++] = new Guid(uid);
            }
            events.DeleteEventHandlerAssociations(arr);
        }
        catch (Exception ex)
        {
            MessageBox.Show("Event handler removal failed.\n" + ex.Message);
        }
    }

    private void UpdateEventsConfig(string logFolder, string logAdminEvent, string logFileEveryHour)
    {
        try
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(getOfficeSystemConfig_Path() + "\\"+ CONFIG_FILE);
            XmlNode config = doc.SelectSingleNode("configuration");
            XmlNode appSettings = config.SelectSingleNode("appSettings");
            if (appSettings == null)
            {
                appSettings = config.AppendChild(doc.CreateNode(XmlNodeType.Element, "appSettings", ""));
            }
            XmlNodeList items = appSettings.SelectNodes("add");
            bool logFolderFound = false;
            bool logAdminEventFound = false;
            bool logFileEveryHourFound = false;
            foreach (XmlNode item in items)
            {
                switch (item.Attributes["key"].Value)
                {
                    case "LogFolderPath":
                        item.Attributes["value"].Value = logFolder;
                        logFolderFound = true;
                        break;
                    case "LogAdminCubeEvent":
                        item.Attributes["value"].Value = logAdminEvent;
                        logAdminEventFound = true;
                        break;
                    case "LogFileEveryHour":
                        item.Attributes["value"].Value = logFileEveryHour;
                        logFileEveryHourFound = true;
                        break;

                }
            }
            if (!logFolderFound)
            {
                SetAttribute(doc, appSettings, "LogFolderPath", logFolder);
            }
            if (!logAdminEventFound)
            {
                SetAttribute(doc, appSettings, "LogAdminCubeEvent", logAdminEvent);
            }
            if (!logFileEveryHourFound)
            {
                SetAttribute(doc, appSettings, "LogFileEveryHour", logFileEveryHour);
            }
            doc.Save(getOfficeSystemConfig_Path() + "\\" + CONFIG_FILE);
        }
        catch (Exception ex)
        {
            MessageBox.Show("UpdateEventsConfig failed.  " + ex.Message);
        }
    }

    private void SetAttribute(XmlDocument doc, XmlNode appSettings, string key, string value)
    {
        XmlNode node = appSettings.AppendChild(doc.CreateNode(XmlNodeType.Element, "add", ""));
        XmlAttribute attr = doc.CreateAttribute("key");
        attr.Value = key;
        node.Attributes.Append(attr);
        XmlAttribute attr1 = doc.CreateAttribute("value");
        attr1.Value = value;
        node.Attributes.Append(attr1);
    }

    private string GetInstallDir()
    {
        string dir = Context.Parameters["AssemblyPath"];
        int index = dir.LastIndexOf('\\');
        if (index == -1) throw new InstallException("Invalid installation path.");
        return dir.Substring(0, index);
    }

    private int GetEventID(string recClass, string eventMethod)
    {
        // Get all the events to filter.
        Installer.wsEvents.EventsDataSet dsEvents = events.ReadEventsList();

        int index = recClass.LastIndexOf("EventReceiver");
        string sourceName = recClass.Substring(0, index);
        string eventName = eventMethod.Substring(2);
        
        DataView dv = new DataView(dsEvents.Event);
        dv.RowFilter = dsEvents.Event.Columns["SourceName"].ColumnName + " = '" + sourceName + "'";
        dv.RowFilter = dv.RowFilter + " AND " + dsEvents.Event.Columns["EventName"].ColumnName + " = '" + eventName + "'";
        if (dv.Count == 0) return -1;
        DataRowView drv = dv[0];
        return (int)drv["EventId"];
    }
}
